##
#  Controller to manage api requests involving repositories
#
class Api::V1::RepositoryController < ApplicationController

  before_action :authenticate_with_soap_token, :set_repo_privileges

  def list_repos_by_project
    fail Exceptions::PermissionDenied unless @can_view_repo
    begin
      repositories = Repository.where(project_url_name:@ctf_project_path,is_deleted:false)
      .select([:id,
               :repository_id,
               :repository_format,
               :repository_type,
               :repository_policy,
               :repository_host,
               :repository_provider
              ])
      render json: repositories.as_json,
             status: 200
    rescue Exceptions::PermissionDenied
      render json: {errors:[ {status: 403, message: "Permission Denied"} ]},
             status: 403
    rescue => e
      render json: {errors:[ {status: 500, message: "Internal server error #{e.message}"} ]},
             status: 500
    end
  end

  def list_repos_by_proj_and_provider
    fail Exceptions::PermissionDenied unless @can_view_repo
    begin
      repositories = Repository.where(project_url_name:@ctf_project_path,
                                      repository_provider:format('%sRepository',params['repo_provider'].capitalize),
                                      is_deleted:false)
      .select([:id,
               :repository_id,
               :repository_format,
               :repository_type,
               :repository_policy,
               :repository_host,
               :repository_provider
              ])
      render json: repositories.as_json,
             status: 200
    rescue Exceptions::PermissionDenied
      render json: {errors:[ {status: 403, message: "Permission Denied"} ]},
             status: 403
    rescue => e
      render json: {errors:[ {status: 500, message: "Internal server error #{e.message}"} ]},
             status: 500
    end

  end

  def render_unauthorized
    render json:{errors:[ {status: 401, message: "Unauthorized"} ]},
           status: 401
  end

end
