##
# class controlling object id based urls
#
class JumptoidController < ApplicationController
  before_action :authenticate, :set_locale

  def redirect_to_object
    repository, path = parse_object_id(params[:object_id])
    binary_app = app_by_name(repository.iaf_app_name)
    fail Exceptions::NoSuchObjectId if binary_app.nil?
    redirect_to repository.repo_url(one_time_token, binary_app['baseurl'], path)
  end

  private

  def parse_object_id(object_id)
    # TODO: do permission check after artf194368 is done
    path = nil
    if object_id =~ /^\d+$/
      repository = Repository.find_by(id: object_id.to_i, is_deleted: false)
    else
      decoded_str = Base64.decode64(object_id)
      url, path = decoded_str.split(':')
      repository = Repository.find_by(repository_host: URI.decode(url),
                                      is_deleted: false)
    end
    fail Exceptions::NoSuchObjectId if repository.blank?
    [repository, path]
  end

  def app_by_name(name)
    Services::SoapService.get_binary_app_by_name(@ctf_soapsession_id, name)
  end
end
