##
# class to decide on permissions
#
class Services::AuthorizationService
  VIEW_ALL = 'View Repositories / Packages'
  CREATE_REPO = 'View / Create Repositories'
  UPDATE_REPO = 'View / Update Repositories'
  DELETE_REPO = 'View / Delete Repositories'
  CREATE_PKG = 'View / Create Packages'
  UPDATE_PKG = 'View / Update Packages'
  DELETE_PKG = 'View / Delete Packages'

  def self.read_repo_permission_in_project?(ctf_soapsession_id, project_id,
                                            permissions = nil)
    permissions ||= get_permissions(ctf_soapsession_id, project_id)
    (permissions & [VIEW_ALL, CREATE_REPO, UPDATE_REPO, DELETE_REPO,
                    CREATE_PKG, UPDATE_PKG, DELETE_PKG]).present?
  end

  def self.create_repo_permission_in_project?(ctf_soapsession_id, project_id,
                                              permissions = nil)
    permissions ||= get_permissions(ctf_soapsession_id, project_id)
    permissions.member? CREATE_REPO
  end

  def self.edit_repo_permission_in_project?(ctf_soapsession_id, project_id,
                                            permissions = nil)
    permissions ||= get_permissions(ctf_soapsession_id, project_id)
    permissions.member? UPDATE_REPO
  end

  def self.delete_repo_permission_in_project?(ctf_soapsession_id, project_id,
                                              permissions = nil)
    permissions ||= get_permissions(ctf_soapsession_id, project_id)
    permissions.member? DELETE_REPO
  end

  def self.view_pkg_permission_in_project?(ctf_soapsession_id, project_id,
                                           permissions = nil)
    read_repo_permission_in_project? ctf_soapsession_id, project_id, permissions
  end

  def self.create_pkg_permission_in_project?(ctf_soapsession_id, project_id,
                                             permissions = nil)
    permissions ||= get_permissions(ctf_soapsession_id, project_id)
    permissions.member? CREATE_PKG
  end

  def self.edit_pkg_permission_in_project?(ctf_soapsession_id, project_id,
                                           permissions = nil)
    permissions ||= get_permissions(ctf_soapsession_id, project_id)
    permissions.member? UPDATE_PKG
  end

  def self.delete_pkg_permission_in_project?(ctf_soapsession_id, project_id,
                                              permissions = nil)
    permissions ||= get_permissions(ctf_soapsession_id, project_id)
    permissions.member? DELETE_PKG
  end

  def self.get_permissions(ctf_soapsession_id, project_id)
    Services::SoapService
      .get_binary_app_permissions_by_projectid ctf_soapsession_id, project_id
  end
end
