##
# class to do operations related to artifactory
#
class Services::Integrations::Artifactory

  def self.create_repository(params)
    begin
      iafApp = Services::SoapService.get_binary_app_by_name params[:sessionId], params[:iafappname]
      repo_key= params[:repoKey]
      payload = repo_payload params
      headers = http_headers params
      url = format('%s/api/repositories/%s', iafApp['baseurl'], repo_key)
      resp_msg = RestClient::Resource.new(url,
                                          headers: headers,
                                          verify_ssl: OpenSSL::SSL::VERIFY_NONE).put payload.to_json
      create_groups repo_key, iafApp['baseurl'], headers
      response = Status.new(true,
                            200,
                            message: resp_msg,
                            data: {'contentResourceURI'=>format('%s/%s',iafApp['baseurl'],repo_key),
                                   'repoPolicy'=>find_repo_policy(params[:handleRelease],params[:handleSnapshots]),
                                   'repoType'=> params[:repoType],
                                   'format'=>params[:pckType],
                                   'name'=> repo_key,
                                   'id'=>repo_key})
    rescue RestClient::ResourceNotFound, RestClient::ResponseForException, RestClient::RequestFailed,
        RestClient::Unauthorized => e
      response = rescue_from_request_failed(e)
    rescue Errno::ECONNREFUSED, Errno::EHOSTUNREACH => e
      response = rescue_from_connection_error(e)
    rescue StandardError => e
      response = rescue_from_unknown_error(e)
    end
    response
  end

  def self.list_repositories(params)
    list_repositories_for_project params
  end

  def self.list_repositories_for_project(params)
    begin
      iafApp = Services::SoapService.get_binary_app_by_name params[:sessionId], params[:iafappname]
      resp_msg = RestClient::Resource.new(
          format('%s/api/repositories', iafApp['baseurl']),
          headers:http_headers(params),
          verify_ssl: OpenSSL::SSL::VERIFY_NONE
      ).get
      repos = JSON.parse(resp_msg)
      response = Status.new(true,
                            200,
                            message: 'Repositories List obtained',
                            data: get_repos_list(repos))
    rescue RestClient::RequestFailed, RestClient::Unauthorized => e
      response = rescue_from_request_failed(e)
    rescue Errno::ECONNREFUSED, Errno::EHOSTUNREACH => e
      response = rescue_from_connection_error(e)
    rescue StandardError => e
      response = rescue_from_unknown_error(e)
    end
    response
  end

  def self.link_repository(params)
    begin
      iafApp = Services::SoapService.get_binary_app_by_name params[:sessionId], params[:iafappname]
      resp = RestClient::Resource.new(
          format('%s/api/repositories/%s', iafApp['baseurl'],params['repoId']),
          headers:http_headers(params),
          verify_ssl: OpenSSL::SSL::VERIFY_NONE
      ).get
      repo = JSON.parse(resp)
      create_groups params['repoId'], iafApp['baseurl'], http_headers(params)
      response = Status.new(true,
                            200,
                            data: {'contentResourceURI'=>format('%s/%s',iafApp['baseurl'],params['repoId']),
                                   'repoPolicy'=>find_repo_policy(repo['handleRelease'],repo['handleSnapshots']),
                                   'repoType'=> repo['rclass'],
                                   'format'=>repo['packageType'],
                                   'name'=> repo['key'],
                                   'id'=>repo['key']})
    rescue RestClient::ResourceNotFound, RestClient::RequestFailed,
        RestClient::Unauthorized => e
      response = rescue_from_request_failed(e)
    rescue Errno::ECONNREFUSED, Errno::EHOSTUNREACH => e
      response = rescue_from_connection_error(e)
    rescue StandardError => e
      response = rescue_from_unknown_error(e)
    end
    response
  end

  protected

  def self.create_groups(repo_key, base_url,headers)
    group_list=["#{repo_key}-read-group","#{repo_key}-deploy-group",
                "#{repo_key}-annotate-group","#{repo_key}-delete-group","#{repo_key}-manage-group"]
    group_list.each do |group_name|
      begin
        payload = group_payload group_name
        resp_msg = RestClient::Resource.new(
            format('%s/api/security/groups/%s', base_url, group_name),
            headers: headers,
            verify_ssl: OpenSSL::SSL::VERIFY_NONE).put payload.to_json
        create_permission repo_key,base_url,group_name,headers
        Rails.logger.info " #{resp_msg}"
      rescue RestClient::ResourceNotFound, RestClient::RequestFailed,
          RestClient::Unauthorized => e
        response = rescue_from_request_failed(e)
      rescue Errno::ECONNREFUSED, Errno::EHOSTUNREACH => e
        response = rescue_from_connection_error(e)
      rescue StandardError => e
        response = rescue_from_unknown_error(e)
      end
    end
  end

  def self.create_permission(repo_key, base_url,group_name,headers)
    begin
      payload = permission_payload group_name,repo_key
      perm_name = find_permission_name group_name,repo_key
      resp_msg = RestClient::Resource.new(
          format('%s/api/security/permissions/%s', base_url, perm_name),
          headers: headers,
          verify_ssl: OpenSSL::SSL::VERIFY_NONE).put payload.to_json
      Rails.logger.info " #{resp_msg}"
    rescue RestClient::ResourceNotFound, RestClient::RequestFailed,
        RestClient::Unauthorized => e
      response = rescue_from_request_failed(e)
    rescue Errno::ECONNREFUSED, Errno::EHOSTUNREACH => e
      response = rescue_from_connection_error(e)
    rescue StandardError => e
      response = rescue_from_unknown_error(e)
    end
  end

  def self.repo_payload(params)
    case params['repoType']
      when 'local'
        {
            rclass:params[:repoType],
            packageType:params[:pckType],
            description:params[:pubDescription],
            notes:params[:inDescription],
            includesPattern:params[:includePatterns],
            excludesPattern:params[:excludePatterns],
            repoLayoutRef:params[:repoLayout],
            checksumPolicyType:params[:checkSumPolicy],
            handleReleases:params[:handleRelease]?true:false,
            handleSnapshots:params[:handleSnapshots]?true:false,
            maxUniqueSnapshots:params[:maxUniqueSnapshots].to_s.to_i,
            snapshotVersionBehavior:params[:mvnSnapshotBehavior],
            suppressPomConsistencyChecks: params[:suppressPOMCheck]?true:false
        }
      when 'remote'
        {
            rclass: params[:repoType],
            packageType:params[:pckType],
            url: params[:url],
            username: params[:remoteUsername],
            password: params[:remotePassword],
            description: params[:pubDescription],
            notes: params[:inDescription],
            includesPattern:params[:includePatterns],
            excludesPattern: params[:excludePatterns],
            remoteRepoChecksumPolicyType: params[:checkSumPolicy],
            handleReleases: params[:handleRelease]?true:false,
            handleSnapshots: params[:handleSnapshots]?true:false,
            maxUniqueSnapshots: params[:maxUniqueSnapshots].to_s.to_i,
            suppressPomConsistencyChecks: params[:suppressPOMCheck]?true:false,
            offline: params[:offline]?true:false ,
            blackedOut: params[:blackedOut]?true:false,
            storeArtifactsLocally: params[:storeArtifactsLocally]?true:false,
            retrievalCachePeriodSecs: params[:retrievalCachePeriod],
            missedRetrievalCachePeriodSecs: params[:missedRetrievalCachePeriod],
            unusedArtifactsCleanupEnabled: params[:handleRelease]?true:false,
            unusedArtifactsCleanupPeriodHours: params[:unusedArtifactsCleanupPeriod],
            fetchJarsEagerly: params[:eagerlyFetchJars]?true:false,
            fetchSourcesEagerly: params[:eagerlyFetchSources]?true:false,
            synchronizeProperties: params[:synchronizeProperties]?true:false,
        }
      when 'virtual'
        {
            rclass: params[:repoType],
            packageType:params[:pckType],
            repositories:params[:repositories],
            description: params[:pubDescription],
            notes:params[:inDescription],
            includesPattern: params[:includePatterns],
            excludesPattern: params[:excludePatterns],
            artifactoryRequestsCanRetrieveRemoteArtifacts: params[:artifactoryReqCanRetrieveRemoteArtifacts]?true:false,
            pomRepositoryReferencesCleanupPolicy: params[:cleanupRepoReferencesinPOMs]
        }
    end
  end

  def self.group_payload(group_name)
    {
        name:group_name,
        description: "Teamforge specific RBAC groups",
        realm: "teamforgeUserCredentialRealm"
    }
  end

  def self.permission_payload(group_name,repo_key)
    if group_name =~ /read/
      {name:"#{repo_key}-readPerm",repositories:[repo_key],principals:{groups:{"#{group_name}"=>["r"]}}}
    elsif group_name =~ /deploy/
      {name:"#{repo_key}-deployPerm",repositories:[repo_key],principals:{groups:{"#{group_name}"=>["w"]}}}
    elsif group_name =~ /annotate/
      {name:"#{repo_key}-annotatePerm",repositories:[repo_key],principals:{groups:{"#{group_name}"=>["n"]}}}
    elsif group_name =~ /delete/
      {name:"#{repo_key}-deletePerm",repositories:[repo_key],principals:{groups:{"#{group_name}"=>["d"]}}}
    elsif group_name =~ /manage/
      {name:"#{repo_key}-managePerm",repositories:[repo_key],principals:{groups:{"#{group_name}"=>["m"]}}}
    end
  end

  def self.find_permission_name(group_name,repo_key)
    if group_name =~ /read/
      "#{repo_key}-readPerm"
    elsif group_name =~ /deploy/
      "#{repo_key}-deployPerm"
    elsif group_name =~ /annotate/
      "#{repo_key}-annotatePerm"
    elsif group_name =~ /delete/
      "#{repo_key}-deletePerm"
    elsif group_name =~ /manage/
      "#{repo_key}-managePerm"
    end
  end

  def self.http_headers(params)
    {
        content_type: :json,
        Authorization: 'Basic '+ encode_credentials(params[:username],params[:sessionId])
    }
  end

  def self.find_repo_policy(handleRelease,handleSnapshot)
    if handleSnapshot && handleRelease
      'Both'.upcase
    elsif handleRelease
      'Release'.upcase
    elsif handleSnapshot
      'Snapshot'.upcase
    else
      ''
    end
  end

  def self.get_repos_list(repos)
    repo_list = []
    repos.each { |repo|
      new_data = {'contentResourceURI'=>repo['url'],
                  'repoPolicy'=>'',
                  'repoType'=> repo['type'],
                  'format'=>'',
                  'name'=> repo['key'],
                  'id'=>repo['key'] }
      repo_list.push new_data
    }
    repo_list
  end

  def self.encode_credentials(user_name,password)
    Base64.strict_encode64 user_name+':'+password
  end

  def self.rescue_from_request_failed(e)
    begin
      if e.response
        error_json = JSON.parse(e.response)
        Status.new(false, e.http_code, message: error_json['errors'][0]['message'])
      else
        Status.new(false, e.http_code, message: e.message)
      end
    rescue JSON::ParserError
      Status.new(false, e.http_code, message: e.response)
    end
  end

  def self.rescue_from_connection_error(e)
    Status.new(
        false,
        503,
        message: 'Error! Probably the Binary server is down.
                  Please contact Teamforge administrator.
                  The error message is as follows ...
                  <br/>' + e.to_s)
  end

  def self.rescue_from_unknown_error(e)
    Status.new(
        false,
        500,
        message: 'Error! An unexpected error occurred.
                  The error message is as follows ...
                  <br/>' + e.to_s)
  end

end
