##
# class to abstract some soap tasks which involves more than 2 soap calls
# and that are very frequently used in many places
#
class Services::SoapService
  def self.list_of_binaries(ctf_soapsession_id)
    ctf_websession_id = TeamForge::GetUserSessionBySoapId
      .new(ctf_soapsession_id).request
    iafsoapservice = TeamForge::IAFService
    list_of_generic_apps = iafsoapservice::ListPluggableComponentsByCategory
      .new(ctf_websession_id,
           :generic).request
    category = nil
    JSON.parse(list_of_generic_apps).each do |generic_app|
      if generic_app['name'] == Rails.configuration.teamforge[:iaf_app_name]
        category = generic_app['category']
      end
      break unless category.nil?
    end
    TeamForge::IAFService::ListPluggableComponentsByCategory
      .new(ctf_websession_id,
           category)
      .request
  end

  def self.get_binary_app_by_name(ctf_soapsession_id, binary_app_name)
    list_of_binaries = list_of_binaries ctf_soapsession_id
    JSON.parse(list_of_binaries).find do |binary_app|
      binary_app['name'] == binary_app_name
    end
  end

  def self.get_binary_app_permissions_by_projectid(ctf_soapsession_id,
                                                   project_id)
    project_path = TeamForge::GetProjectData
      .new(ctf_soapsession_id, project_id).request['path']
    pluggable_app_soap = TeamForge::PluggableApp
    base_app_prplid =  pluggable_app_soap::GetLinkPlugId
      .new(ctf_soapsession_id, project_path,
           Rails.configuration.teamforge[:iaf_base_url])
      .request
    TeamForge::RbacApp::GetIntegratedAppPermissionsForProject
      .new(ctf_soapsession_id, project_id, base_app_prplid)
      .request
  end

  def self.get_binary_app_prefix(ctf_soapsession_id)
    ctf_websession_id = TeamForge::GetUserSessionBySoapId
      .new(ctf_soapsession_id).request
    iafsoapservice = TeamForge::IAFService
    list_of_generic_apps = iafsoapservice::ListPluggableComponentsByCategory
    .new(ctf_websession_id,
         :generic).request
    app = JSON.parse(list_of_generic_apps).find do |generic_app|
      generic_app['name'] == Rails.configuration.teamforge[:iaf_app_name]
    end
    app && app['prefix']
  end

  def self.get_endpoint_url_for_iaf_app(ctf_soapsession_id, iaf_app_name)
    list_of_binaries = list_of_binaries ctf_soapsession_id
    app = JSON.parse(list_of_binaries).find do |binary_app|
      binary_app['name'] == iaf_app_name
    end
    app && app['endpoint']
  end
end
