Rails.application.routes.draw do
  # we follow the convention that all urls will have this prefix /iaf/binary to indicate two things
  #1. it is an iaf app
  #2. it is a binary type multiplexer
  app_url_prefix = Rails.configuration.teamforge.fetch 'app_url_prefix' rescue ''
  scope(:path => "/#{app_url_prefix}") do

    # 1. repositories
    get '/listRepositories' => 'repository#show'
    get '/' => 'repository#show', :constraints  => { :sfProj => /[^\/]+/ }
    get '/projects.:sfProj' => 'repository#show', :constraints  => { :sfProj => /[^\/]+/ }
    get '/createRepository' => 'repository#create', :constraints  => { :sfProj => /[^\/]+/ }
    get '/createRepository/projects.:sfProj' => 'repository#create', :constraints  => { :sfProj => /[^\/]+/ }
    post '/createRepository/projects.:sfProj' => 'repository#create', :constraints  => { :sfProj => /[^\/]+/ }
    post '/deleteRepository/projects.:sfProj' => 'repository#destroy', :constraints  => { :sfProj => /[^\/]+/ }
    get '/listLinkableRepositories/projects.:sfProj' => 'repository#show_linkable', :constraints  => { :sfProj => /[^\/]+/ }
    post '/linkRepository/projects.:sfProj' => 'repository#link', :constraints  => { :sfProj => /[^\/]+/ }
    get '/go/:object_id' => 'jumptoid#redirect_to_object'

    # 2. reports
    resources :report

    namespace :services do
      namespace :team_forge , defaults: { format: :xml } do
        get '/ConfigurationParameters/:token/:tf_project_id' => :get_configuration_parameters
        post '/ProjectConfiguration/:token' => :create_project_configuration
        put '/ProjectConfiguration/:token' => :update_project_configuration
        delete '/ProjectConfiguration/:token/:tf_project_id' => :delete_project_configuration
        get '/ProjectId/:token/:object_id' => :get_ctf_project_id
        get '/Title/:token/:object_id' => :get_title

      end
    end

    # 2. REST API
    namespace :api do
      namespace :v1 do
        scope :projects do
          get '/'=> 'project#show_all'
        end
        scope :repositories do
          get '/:tf_project_path' => 'repository#list_repos_by_project', :constraints  => { :tf_project_path =>  /[^\/]+/ }
          get '/:tf_project_path/provider/:repo_provider' =>'repository#list_repos_by_proj_and_provider',:constraints =>{ :tf_project_path =>  /[^\/]+/ ,
                        :repo_provider=>/[^\/]+/}
        end
      end
    end
  end
end
