require 'spec_helper'
require 'savon'
describe RepositoryController do
  app_url_prefix = Rails.configuration.teamforge['app_url_prefix']
  describe '#show' do
    context 'when user clicks the Binary button in the teamforge button bar' do
      it 'he is redirected to flushed url' do
        # url being mocked
        # base_url?sfLoginToken=JHSD9123SDJ&sfProj=sf_proj&isLoggedIn=true
        # input helpers
        request.path_info = "/#{app_url_prefix}"

        # output helpers
        redirect_url = "/#{app_url_prefix}/projects.proj_url"

        savon.expects(:get_session_id)
          .returns(:get_session_id)

        savon.expects(:get_link_plug_id)
          .returns(:get_link_plug_id)

        savon.expects(:get_current_user_data)
          .returns(:get_current_user_data_admin)

        get :show, sfLoginToken: 'ctf_one_time_token', sfProj: 'proj_url',
            isLoggedIn: 'true'
        expect(response).to redirect_to redirect_url
      end
    end

    context 'when the url is used directly after admin has logged into ' +
            'binary app at least once' do
      it 'he can see a list of repositories' do
        # url being mocked base_url/projects.sf_proj
        # in fact it ias base_url/projects.:sf_proj
        # input helpers
        request.path_info = "/#{app_url_prefix}/projects.sf_proj"
        session[:ctf_soapsession_id] = 'session_id'
        # output helpers
        # redirect_url = "/#{app_url_prefix}/projects.sf_proj"

        savon.expects(:get_link_plug_id)
          .returns(:get_link_plug_id)

        savon.expects(:get_current_user_data)
          .returns(:get_current_user_data_admin)

        savon.expects(:get_integrated_app_permissions_for_project)
          .returns(:get_integrated_app_permissions_for_project_admin)

        get :show, sfProj: 'proj_url'
        # expect(response).to redirect_to redirect_url
      end
    end

    context 'when the url is used directly after user with view repositories ' +
            '/ packages has logged into binary app at least once' do
      it 'he can see a list of repositories' do
        # url being mocked base_url/projects.sf_proj
        # in fact it ias base_url/projects.:sf_proj
        # input helpers
        request.path_info = "/#{app_url_prefix}/projects.sf_proj"
        session[:ctf_soapsession_id] = 'session_id'
        # output helpers
        # redirect_url = "/#{app_url_prefix}/projects.sf_proj"

        savon.expects(:get_link_plug_id)
        .returns(:get_link_plug_id)

        savon.expects(:get_current_user_data)
        .returns(:get_current_user_data_admin)

        savon.expects(:get_integrated_app_permissions_for_project)
        .returns(:perm_1)

        get :show, sfProj: 'proj_url'
        # expect(response).to redirect_to redirect_url
      end
    end
  end
end
