require "savon"
require "time"
require "uri"
require "teamforge/ctfsoap"
require "teamforge/version"
require "teamforge/tfsoaprequest"
require "teamforge/tftype"
require "teamforge/services/collabnet"



module TeamForge  
  
  SERVICE_NAMESPACE="http://schema.open.collab.net/sfee50/soap60/service"
  TYPE_NAMESPACE="http://schema.open.collab.net/sfee50/soap60/type"
  SOAP_NAMESPACE="http://schemas.xmlsoap.org/soap/encoding/"
  SERVICE_PATH="/ce-soap60/services/"
  
 
  autoload :TrackerApp, "teamforge/services/trackerapp"
  autoload :WikiApp, "teamforge/services/wikiapp"
  autoload :FrsApp, "teamforge/services/frsapp"
  autoload :ScmApp, "teamforge/services/scmapp"
  autoload :ScmListener, "teamforge/services/scmlistener"
  autoload :IAFService, "teamforge/services/iafservice"
  autoload :PlanningApp, "teamforge/services/planningapp"
  autoload :DiscussionApp, "teamforge/services/discussionapp"
  autoload :CategorizationApp, "teamforge/services/categorizationapp"
  autoload :DocumentApp, "teamforge/services/documentapp"
  autoload :RbacApp, "teamforge/services/rbacapp"
  autoload :SimpleFileStorageApp, "teamforge/services/simplefilestorageapp"
  autoload :IntegrationDataApp, "teamforge/services/integrationdataapp"
  autoload :PluggableApp, "teamforge/services/pluggableapp"
  autoload :PageApp, "teamforge/services/pageapp"
  autoload :NewsApp, "teamforge/services/newsapp"

  def self.proxy
    @proxy
  end
  
  def self.proxy=(proxy)
    @proxy=format_url(proxy)
  end
  
  def self.server
    @server
  end
  
  def self.server=(server)
    @server=format_url(server, @use_ssl)
  end
  
  def self.verbose?
    @verbose
  end
  
  def self.verbose=(verbose)
    if !!verbose == true && verbose != 0
      @verbose=true
    else
      @verbose=false
    end
  end
  
  def self.use_ssl
    @use_ssl
  end
  
  def self.use_ssl=(use_ssl)
    @use_ssl = use_ssl
  end
  
  def self.format_url(url, ssl=nil)
   
    @use_ssl.nil? ? @use_ssl = true : @use_ssl
    
    ssl.nil? ? ssl = @use_ssl : @use_ssl = ssl
    
    if ssl 
      url = "https://#{url.gsub(/^http(|s):\/\/?/i,'')}"
    else
      url = "http://#{url.gsub(/^http(|s):\/\/?/i,'')}"
    end
    
    uri = URI.parse url
    unless uri.kind_of? URI::HTTP
      raise TeamForgeError "Invalid Server URL"
    end
        
    return url
    
  end

  # Currently, this SDK wrapper uses Savon to send and receive soap messages. Only the Client is being used currently but not any of the parsers. Adapting to a different HTTP library would be fairly simple. 
  # When loaded, Savon logging is turned off as is HTTPI logging. Otherwise, by default Savon writes all messages to Stdout.
  
  
  
  # Error to indicate problems accessing a CTF server
  class TeamForgeError < RuntimeError

  	attr_accessor :request

  	def initialize(msg=nil, request=nil)
  	  super(msg)
  		@request = request
  	end
  end
  

end

   
