##
# Class to post messeges to orchestrate
class OrchestrateClient
  private_class_method :new # force callers to use connect

  attr_accessor :connection, :queue, :exchange, :connection_string

  def initialize(server, user, pass)
    # accept abc.com, amqp://abc.com, amqp://abc.com:5678, or abc.com:5678
    matched = server.match %r{\A(amqp://)?([^:]*)(:(\d+))?\z}
    proto, host, _, port = matched[1, 4]
    proto ||= 'amqp://'
    port ||= 5672
    self.connection_string = "#{proto}#{user}:#{pass}@#{host}:#{port}"
  end

  def self.connect(server, user, pass, queue_name)
    obj = new(server, user, pass)
    obj.connection = Bunny.new(obj.connection_string, connection_timeout: 10)
    obj.connection.start
    channel = obj.connection.create_channel
    obj.queue = channel.queue(queue_name, durable: true)
    obj.exchange = channel.default_exchange

    return obj unless block_given?

    begin
      yield obj
    ensure
      obj.close
    end
  end

  def post(body)
    exchange.publish(body, routing_key: queue.name)
  end

  def close
    connection.close
  end
end
